(***********************************************************************
************************************************************************
****                                                                ****
****                                                                ****
****     File Name: untRAS.pas                                      ****
****                                                                ****
****     Author: //hIDRA_5                                          ****
****                                                                ****
****     Description: Procedures to manipulate RAS entries          ****
****                                                                ****
****                                                                ****
************************************************************************
************************************************************************)

unit untRAS;

interface

uses Classes, Windows, SysUtils;

{ declarations }

const
  DNLEN = 15;  // Maximum domain name length
  UNLEN = 256; // Maximum user name length
  PWLEN = 256; // Maximum password length

  RAS_MaxDeviceType  = 16;
  RAS_MaxPhoneNumber = 128;

{$IFDEF WINVER31}
  RAS_MaxEntryName      = 20;
  RAS_MaxDeviceName     = 32;
  RAS_MaxCallbackNumber = 48;
{$ELSE}
  RAS_MaxEntryName      = 256;
  RAS_MaxDeviceName     = 128;
  RAS_MaxCallbackNumber = RAS_MaxPhoneNumber;
{$ENDIF}


type
  TRasEntryNameA = record
    dwSize: Longint;
    szEntryName: Array[0..RAS_MaxEntryName] of AnsiChar;
  end;

  LPRasEntryName = ^TRasEntryName;

  TRasEntryName = TRasEntryNameA;

  LPRasConnA = ^TRasConnA;
  LPRasConn = ^THRasConn;
  THRasConn = Longint;  

  TRasConnA = record
    dwSize: Longint;
    hrasconn: THRasConn;
    szEntryName: Array[0..RAS_MaxEntryName] of AnsiChar;
{$IFNDEF WINVER31}
   szDeviceType: Array[0..RAS_MaxDeviceType] of AnsiChar;
   szDeviceName: Array[0..RAS_MaxDeviceName] of AnsiChar;
{$ENDIF}
{$IFDEF WINVER41}
    szPhonebook: Array[0..MAX_PATH - 1] of AnsiChar;
    dwSubEntry: Longint;
{$ENDIF}
  end;

  TRasConn = TRasConnA;

  TRasDialParamsA = record
    dwSize: LongInt;
    szEntryName: Array[0..RAS_MaxEntryName] of AnsiChar;
    szPhoneNumber: Array[0..RAS_MaxPhoneNumber] of AnsiChar;
    szCallbackNumber: Array[0..RAS_MaxCallbackNumber] of AnsiChar;
    szUserName: Array[0..UNLEN] of AnsiChar;
    szPassword: Array[0..PWLEN] of AnsiChar;
    szDomain: Array[0..DNLEN] of AnsiChar;
{$IFDEF WINVER41}
    dwSubEntry: Longint;
    dwCallbackId: Longint;
{$ENDIF}
  end;

  TRasDialParams = TRasDialParamsA;

function RasEnumEntries(
    reserved: PChar;
    lpszPhoneBook: PChar;
    entrynamesArray: LPRasEntryName;
    var lpcb: Longint;
    var lpcEntries: Longint
    ): Longint; stdcall;

function RasRenameEntry(
    lpszPhonebook,
    szEntryOld,
    szEntryNew: PChar
    ): Longint; stdcall;

function RasDeleteEntry(
    lpszPhonebook,
    szEntry: PChar
    ): Longint; stdcall;    

function RasEditPhonebookEntry(
    hwndParentWindow: HWND;
    lpszPhoneBook: PChar;
    lpszEntryName: PChar
    ): Longint; stdcall;

function RasHangUp(
    hConn: THRasConn
    ): Longint; stdcall;

function RasCreatePhonebookEntry(
    hwndParentWindow: HWND;
    lpszPhoneBook: PChar
    ): Longint; stdcall;

function RasEnumConnections(
    rasconnArray: LPRasConn;
    var lpcb: Longint;
    var lpcConnections: Longint
    ): Longint; stdcall;

function RasGetEntryDialParams(
    lpszPhoneBook: PChar;
    var lpDialParams: TRasDialParams;
    var lpfPassword: LongBool
    ): Longint; stdcall;    

function RasEnumEntries; external 'rasapi32.dll' name 'RasEnumEntriesA';
function RasRenameEntry; external 'rasapi32.dll' name 'RasRenameEntryA';
function RasDeleteEntry; external 'rasapi32.dll' name 'RasDeleteEntryA';
function RasEditPhonebookEntry; external 'rasapi32.dll' name 'RasEditPhonebookEntryA';
function RasHangUp; external 'rasapi32.dll' name 'RasHangUpA';
function RasCreatePhonebookEntry; external 'rasapi32.dll' name 'RasCreatePhonebookEntryA';
function RasEnumConnections; external 'rasapi32.dll' name 'RasEnumConnectionsA';
function RasGetEntryDialParams; external 'rasapi32.dll' name 'RasGetEntryDialParamsA';

{ procs,funcs }

procedure EnumEntries(Items:TStrings);
procedure RenameEntry(OldName, NewName:string);
procedure DeleteEntry(EntryName:string);
procedure ShowEditDlg(EntryName:string);
procedure HangUp;
procedure ShowNewConnection;
procedure GetRasPwds(var Pass:string);
function EntryInfo(EntryName:string):string;

implementation

procedure EnumEntries(Items:TStrings);
var
   Entries:array[1..256] of TRasEntryName;
   BufSize,Num,i:Integer;
begin
   Entries[1].dwSize:=SizeOf(TRasEntryName);
   BufSize:=SizeOf(TRasEntryName) * 256;
   Items.Clear;
   if RasEnumEntries(nil,nil,@Entries[1],BufSize,Num) = 0 then
      if Num > 0 then
         for i:=1 to Num do
            Items.Add(Entries[i].szEntryName);
end;

procedure RenameEntry(OldName, NewName:string);
begin
   RasRenameEntry(nil,PChar(OldName),PChar(NewName));
end;

procedure DeleteEntry(EntryName:string);
begin
   RasDeleteEntry(nil,PChar(EntryName));
end;

procedure ShowEditDlg(EntryName:string);
begin
   RasEditPhoneBookEntry(0,PChar(EntryName),PChar(EntryName));
end;

procedure HangUp;
var
   Conns:array [1..256] of TRasConn;
   BufSize,Num,i:Integer;
begin
   Conns[1].dwSize:=SizeOf(TRasConn);
   BufSize:=SizeOf(TRasConn) * 256;
   if RasEnumConnections(@Conns[1],BufSize,Num) = 0 then
      if Num > 0 then
         for i:=1 to Num do
            RasHangUp(Conns[i].hrasconn);
end;

procedure ShowNewConnection;
begin
   RasCreatePhoneBookEntry(0,nil);
end;

procedure GetRasPwds(var Pass:string);
var
   Items:TStrings;
   i:Integer;
begin
   Pass:='';
   Items:=TStringList.Create;
   EnumEntries(Items);
   for i:=0 to Items.Count - 1 do
      Pass:=Pass + EntryInfo(Items.Strings[i]) + #13#10#13#10;
   Delete(Pass,Length(Pass) - 3,4);
   FreeAndNil(Items);
end;

function EntryInfo(EntryName:string):string;
var
   DPrms:TRasDialParams;
   b:LongBool;   
begin
   DPrms.dwSize:=SizeOf(TRasDialParams);
   FillChar(DPrms.szEntryName,Length(EntryName),0);
   StrPCopy(DPrms.szEntryName,EntryName);
   RasGetEntryDialParams(PChar(EntryName),DPrms,b);
   EntryInfo:='[Connection: ' + EntryName + ']' + #13#10 +
              'Login: ' + DPrms.szUserName + #13#10 +
              'Password: ' + DPrms.szPassword;
end;

end.
