(***********************************************************************
************************************************************************
****                                                                ****
****                                                                ****
****     File Name: untTransfer.pas                                 ****
****                                                                ****
****     Author: //hIDRA_5                                          ****
****                                                                ****
****     Description: Unit main of Transfer form                    ****
****                                                                ****
****                                                                ****
************************************************************************
************************************************************************)

unit untTransfer;

interface

uses
  Windows, SysUtils, Classes, Controls, Forms, ComCtrls, StdCtrls, ScktComp,
  Buttons, ImgList, TypInfo;

type
  TTransferAction = (taDownload, taUpload);

  PTransferInfo = ^TTransferInfo;
  TTransferInfo = record
    LocalFile,
    RemoteFile  : string;
    TotalSize,
    CurrentSize : Integer;
    StartTime   : LongWord;
    Action      : TTransferAction;
    Ready,
    Error,
    Done        : Boolean;
    TextFile    : TextFile;
    BinFile     : file of Char;
    Client      : TClientSocket;
    Item        : TListItem;
  end;

  TfrmTransfer = class(TForm)
    SpdCancel: TSpeedButton;
    LvwTransfer: TListView;
    Label1: TLabel;
    Label2: TLabel;
    EdtLocal: TEdit;
    EdtRemote: TEdit;
    SpdClean: TSpeedButton;
    ImgsTransfer: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure DoTransfer(LocalFile, RemoteFile:string; Action: TTransferAction);
    procedure ClientTransferConnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ClientTransferRead(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure SpdCancelClick(Sender: TObject);
    procedure ClientTransferDisconnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ClientTransferError(Sender: TObject;
      Socket: TCustomWinSocket; ErrorEvent: TErrorEvent;
      var ErrorCode: Integer);
    procedure LvwTransferClick(Sender: TObject);
    procedure FreePointer(Item: TListItem);
    procedure SpdCleanClick(Sender: TObject);
  private
    { Public declarations }
  public
    { Public declarations }
  end;

var
  frmTransfer: TfrmTransfer;

implementation

uses untMain;

{$R *.DFM}

procedure TfrmTransfer.FormCreate(Sender: TObject);
begin
   Caption:='Fenetre de transfert';
   ManualDock(frmMain.PntTransfer,nil,alNone);
   Left:=(frmMain.PntTransfer.Width - Width) div 2;
   Top:=(frmMain.PntTransfer.Height - Height) div 2;
   Show;
end;

procedure TfrmTransfer.DoTransfer(LocalFile, RemoteFile:string; Action: TTransferAction);
var
   PInfo:PTransferInfo;
   tPort,j:Integer;
begin
   tPort:=StrToIntDef(Trim(frmMain.EdtTransfer.Text),-1);
   if (tPort <=0) or (tPort > 65535) or (tPort = frmMain.ClientMain.Port) then
   begin
      MessageBox(Handle,'invalide upload/download port...',PChar(frmMain.Caption),MB_ICONERROR);
      Exit;
   end;
   New(PInfo);
   PInfo^.LocalFile:=LocalFile;
   PInfo^.RemoteFile:=RemoteFile;
   PInfo^.TotalSize:=0;
   PInfo^.CurrentSize:=0;
   PInfo^.StartTime:=GetTickCount;
   PInfo^.Action:=Action;
   PInfo^.Ready:=False;
   PInfo^.Error:=False;
   PInfo^.Done:=False;
   PInfo^.Client:=TClientSocket.Create(Self);
   PInfo^.Client.OnConnect:=ClientTransferConnect;
   PInfo^.Client.OnDisconnect:=ClientTransferDisconnect;
   PInfo^.Client.OnError:=ClientTransferError;
   PInfo^.Client.OnRead:=ClientTransferRead;
   PInfo^.Item:=LvwTransfer.Items.Add;
   PInfo^.Item.Caption:=ExtractFileName(RemoteFile);
   PInfo^.Item.ImageIndex:=Integer(Action);
   for j:=0 to 3 do
      PInfo^.Item.SubItems.Insert(0,'');
   PInfo^.Item.SubItems[2]:='Dmarage ' + LowerCase(Copy(GetEnumName(TypeInfo(TTransferAction),Integer(Action)),2,MaxInt)) + '...';
   try
      PInfo^.Client.Port:=tPort;
      PInfo^.Client.Address:=frmMain.ClientMain.Address;
      PInfo^.Client.Open;
   except
      PInfo^.Item.SubItems[2]:='Erreur de transfert...';
      frmMain.CloseSocket(PInfo^.Client);
      Exit;
   end;
   PInfo^.Client.Socket.Data:=PInfo;   
   PInfo^.Item.Data:=PInfo;
end;

procedure TfrmTransfer.ClientTransferConnect(Sender: TObject;
  Socket: TCustomWinSocket);
var
   PInfo:PTransferInfo;
begin
   PInfo:=Socket.Data;
   try
      if PInfo^.Action = taDownload then
      begin
         AssignFile(PInfo^.TextFile,PInfo^.LocalFile);
         ReWrite(PInfo^.TextFile);
      end
      else if PInfo^.Action = taUpload then
      begin
         AssignFile(PInfo^.BinFile,PInfo^.LocalFile);
         Reset(PInfo^.BinFile);
         PInfo^.TotalSize:=FileSize(PInfo^.BinFile);
      end;
   except
      frmMain.CloseSocket(PInfo^.Client);
      PInfo^.Item.SubItems[2]:='Impossible d''acceder au fichier...';
      Exit;
   end;
   PInfo^.Item.SubItems[2]:=Copy(GetEnumName(TypeInfo(TTransferAction),Integer(Action)),2,MaxInt) + ' started...';
   PInfo^.Client.Socket.SendText(PInfo^.RemoteFile + Chr(13) + IntToStr(Integer(PInfo^.Action)));
end;

procedure TfrmTransfer.ClientTransferRead(Sender: TObject;
  Socket: TCustomWinSocket);
var
   PInfo:PTransferInfo;
   Dta:string;
   Buffer:array [1..256] of Char;
   r,p:LongInt;
begin
   PInfo:=Socket.Data;
   Dta:=Socket.ReceiveText;
   Application.ProcessMessages;
   if (PInfo^.Done) then Exit;
   if PInfo^.Action = taDownload then
   begin
      if PInfo^.Ready then
      begin
         Write(PInfo^.TextFile,Dta);
         PInfo^.CurrentSize:=PInfo^.CurrentSize + Length(Dta);
         PInfo^.Item.SubItems[0]:=Format('%d of %d',[PInfo^.CurrentSize,PInfo^.TotalSize]);
         PInfo^.Item.SubItems[1]:=IntToStr(Trunc(PInfo^.CurrentSize / PInfo^.TotalSize * 100)) + '%';
         PInfo^.Item.SubItems[2]:='Download...';
         PInfo^.Item.SubItems[3]:=FormatFloat('0#.00',(PInfo^.CurrentSize / 1024) / ((GetTickCount - PInfo^.StartTime) / 1000)) + ' k/s';
      end
      else
      begin
         PInfo^.TotalSize:=StrToIntDef(Dta,-1);
         if PInfo^.TotalSize < 0 then
         begin
            PInfo^.Error:=True;
            PInfo^.Item.SubItems[2]:=Dta;
            frmMain.CloseSocket(PInfo^.Client);
            Exit;
         end;
         PInfo^.Ready:=True;
      end;
      if PInfo^.CurrentSize >= PInfo^.TotalSize then
      begin
         PInfo^.Done:=True;
         frmMain.CloseSocket(PInfo^.Client);
         Exit;
      end;
      Application.ProcessMessages;
      PInfo^.Client.Socket.SendText('C');
   end
   else if PInfo^.Action = taUpload then
   begin
      if PInfo^.CurrentSize >= PInfo^.TotalSize then
      begin
         PInfo^.Done:=True;
         frmMain.CloseSocket(PInfo^.Client);
         Exit;
      end;
      if Length(Dta) <> 1 then
      begin
         PInfo^.Error:=True;
         PInfo^.Item.SubItems[2]:=Dta;
         SpdCancelClick(nil);
         Exit;
      end;
      p:=FilePos(PInfo^.BinFile);
      BlockRead(PInfo^.BinFile,Buffer,256,r);
      r:=FileSize(PInfo^.BinFile);
      Dta:=string(Buffer);
      if (r - p) < 256 then
         Dta:=Copy(Dta,1,r - p);
      PInfo^.CurrentSize:=PInfo^.CurrentSize + Length(Dta);
      PInfo^.Item.SubItems[0]:=Format('%d of %d',[PInfo^.CurrentSize,PInfo^.TotalSize]);
      PInfo^.Item.SubItems[1]:=IntToStr(Trunc(PInfo^.CurrentSize / PInfo^.TotalSize * 100)) + '%';
      PInfo^.Item.SubItems[2]:='Upload...';
      PInfo^.Item.SubItems[3]:=FormatFloat('0#.00',(PInfo^.CurrentSize / 1024) / ((GetTickCount - PInfo^.StartTime) / 1000)) + ' k/s';
      Application.ProcessMessages;      
      PInfo^.Client.Socket.SendText(Dta);
   end;
end;

procedure TfrmTransfer.SpdCancelClick(Sender: TObject);
var
   PInfo:PTransferInfo;
begin
   if LvwTransfer.Selected <> nil then
   begin
      PInfo:=LvwTransfer.Selected.Data;
      if (PInfo^.Client.Socket.Connected) then
      begin
         PInfo^.Done:=True;
         while (PInfo^.Client.Socket.Connected) do
         begin
            Application.ProcessMessages;
            frmMain.CloseSocket(PInfo^.Client);
         end;
         ClientTransferDisconnect(nil,PInfo^.Client.Socket);
      end;
   end;
end;

procedure TfrmTransfer.ClientTransferDisconnect(Sender: TObject;
  Socket: TCustomWinSocket);
var
   PInfo:PTransferInfo;
begin
   PInfo:=Socket.Data;
   Application.ProcessMessages;
   if (PInfo^.Error) then
   begin
      PInfo^.Item.SubItems[2]:='Erreur de transfert...';
   end
   else
   begin
      if PInfo^.CurrentSize >= PInfo^.TotalSize then
         PInfo^.Item.SubItems[2]:='Fichier complet...'
      else
         PInfo^.Item.SubItems[2]:='Canceler...'
   end;
   PInfo^.Done:=True;
   PInfo^.Item.SubItems[3]:='';
   PInfo^.Item.SubItems[1]:='';
   try
      if (PInfo^.Action = taDownload) then
         CloseFile(PInfo^.TextFile);
      if (PInfo^.Action = taUpload) then
         CloseFile(PInfo^.BinFile);
   except
   end;
end;

procedure TfrmTransfer.ClientTransferError(Sender: TObject;
  Socket: TCustomWinSocket; ErrorEvent: TErrorEvent;
  var ErrorCode: Integer);
var
   PInfo:PTransferInfo;  
begin
   PInfo:=Socket.Data;
   ErrorCode:=0;
   PInfo^.Error:=False;
   PInfo^.CurrentSize:=-2;
   frmMain.CloseSocket(PInfo^.Client);
   ClientTransferDisconnect(nil,Socket);
end;

procedure TfrmTransfer.LvwTransferClick(Sender: TObject);
var
   PInfo:PTransferInfo;
begin
   Application.ProcessMessages;
   EdtRemote.Clear;
   EdtLocal.Clear;
   if LvwTransfer.Selected <> nil then
   begin
      PInfo:=LvwTransfer.Selected.Data;
      EdtRemote.Text:=PInfo^.RemoteFile;
      EdtLocal.Text:=PInfo^.LocalFile;
   end;
end;

procedure TfrmTransfer.FreePointer(Item: TListItem);
var
   Ptr:Pointer;
   PInfo:PTransferInfo;
begin
   Application.ProcessMessages;
   Ptr:=Item.Data;
   if Ptr <> nil then
   begin
      try
         PInfo:=Ptr;
         PInfo^.Item:=nil;
         PInfo^.Client.Free;
         FreeMem(PInfo);
      finally
         Item.Data:=nil;
      end;
   end;
end;

procedure TfrmTransfer.SpdCleanClick(Sender: TObject);
var
   j:Integer;
begin
   with LvwTransfer do
   begin
      Application.ProcessMessages;
      if (Selected <> nil) and (Selected.Data = nil) then
      begin
         EdtRemote.Clear;
         EdtLocal.Clear;
      end;
      for j:=Items.Count - 1 downto 0 do
      begin
         if (Items.Item[j].Data <> nil) and
            (PTransferInfo(Items.Item[j].Data)^.Done) then
         begin
            FreePointer(Items.Item[j]);
            Items.Delete(j);
         end;
      end;
   end;
end;

end.
